@@ -51,6 +51,8 @@ static wchar_t *expand_ps1_posix(wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
 static wchar_t *expand_ps_yash(wchar_t *s)
     __attribute__((nonnull,malloc,warn_unused_result));
+static inline void skip_alnum(wchar_t **sp)
+    __attribute__((nonnull));
 
 /* An input function that inputs from a multibyte string.
  * `inputinfo' is a pointer to a `struct input_mbs_info'.
@@ -147,6 +149,7 @@ int input_file(struct xwcsbuf_T *buf, void *inputinfo)
     int fd = fileno(f);
     size_t initlen = buf->length;
 
+    block_sigchld_and_sigint();
 start:
     wb_ensuremax(buf, buf->length + 100);
     if (fgetws(buf->contents + buf->length, buf->maxlength - buf->length, f)) {
@@ -179,6 +182,7 @@ start:
 	}
     }
 end:
+    unblock_sigchld_and_sigint();
     return (initlen == buf->length) ? EOF : 0;
 }
 
@@ -214,6 +218,7 @@ bool read_line_from_stdin(struct xwcsbuf_T *buf, bool trap)
 
     if (!set_nonblocking(STDIN_FILENO))
 	return false;
+    block_sigchld_and_sigint();
     while (ok) {
 	char c;
 	ssize_t n = read(STDIN_FILENO, &c, 1);
@@ -253,6 +258,7 @@ bool read_line_from_stdin(struct xwcsbuf_T *buf, bool trap)
 	}
     }
 done:
+    unblock_sigchld_and_sigint();
     unset_nonblocking(STDIN_FILENO);
     return ok;
 }
@@ -444,7 +450,8 @@ wchar_t *expand_ps_yash(wchar_t *s)
 #if YASH_ENABLE_HISTORY
 	case L'!':    wb_wprintf(&buf, L"%d", hist_next_number); break;
 #endif
-	case L'[':    // TODO  expand_ps_yash: \[ and \]
+	case L'f':    skip_alnum(&s);  break;
+	case L'[':    break;
 	case L']':    break;
 	}
 	s++;
@@ -454,6 +461,14 @@ done:
     return wb_towcs(&buf);
 }
 
+void skip_alnum(wchar_t **sp)
+{
+    while (iswalnum(**sp))
+	(*sp)++;
+    if (**sp != L'.')
+	(*sp)--;
+}
+
 /* Sets O_NONBLOCK flag of the specified file descriptor.
  * If `fd' is negative, does nothing.
  * Returns true if successful, or false otherwise, with `errno' set. */
