@@ -50,15 +50,13 @@
 /* About the shell's signal handling:
  *
  * Yash always catches SIGCHLD.
- * When job control is active, SIGTTOU and SIGTSTP are also ignored.
+ * When job control is active, SIGTTOU and SIGTSTP are ignored.
  * If the shell is interactive, SIGINT, SIGTERM and SIGQUIT are also ignored.
  * Other signals are caught if a trap for the signals are set.
  *
  * SIGQUIT and SIGINT are ignored in an asynchronous list.
  * SIGTSTP is ignored in a command substitution in an interactive shell.
  * SIGTTOU is blocked during `tcsetpgrp'.
- * all signals other than SIGPIPE is blocked when writing the contents of a
- * here-document to a pipe.
  * SIGCHLD and SIGINT are blocked to avoid a race condition
  *  - while reading the output of a command substitution, or
  *  - in `wait_for_job'. */
@@ -474,17 +472,6 @@ void unblock_sigttou(void)
 	xerror(errno, "sigprocmask(UNBLOCK, TTOU)");
 }
 
-/* Sets the action of SIGPIPE to the default. */
-void reset_sigpipe(void)
-{
-    struct sigaction action;
-    sigemptyset(&action.sa_mask);
-    action.sa_flags = 0;
-    action.sa_handler = SIG_DFL;
-    if (sigaction(SIGPIPE, &action, NULL) < 0)
-	xerror(errno, "sigaction(SIGPIPE)");
-}
-
 /* Sends SIGSTOP to the shell process.
  * Returns true iff successful. `errno' is set on failure. */
 bool send_sigstop_to_myself(void)
@@ -542,7 +529,7 @@ void unblock_sigchld_and_sigint(void)
  * SIGCHLD and SIGINT must be blocked when this function is called.
  * If SIGCHLD is already caught, this function doesn't wait.
  * If `interruptible' is true, this function can be canceled by SIGINT.
- * If `return_on_trap' is ture, this function returns false immediately after
+ * If `return_on_trap' is true, this function returns false immediately after
  * trap actions are performed. Otherwise, traps are not handled.
  * Returns false iff interrupted. */
 bool wait_for_sigchld(bool interruptible, bool return_on_trap)
@@ -594,8 +581,6 @@ bool wait_for_sigchld(bool interruptible, bool return_on_trap)
  * If SIGCHLD is caught while waiting, `handle_sigchld' is called.
  * SIGCHLD must be blocked when this function is called.
  * If `trap' is true, traps are also handled while waiting. */
-/* Note: some functions call this function without blocking SIGCHLD when the
- * race condition is not a severe problem. */
 void wait_for_input(int fd, bool trap)
 {
     sigset_t ss;
@@ -606,6 +591,12 @@ void wait_for_input(int fd, bool trap)
 	if (trap)
 	    handle_traps();
 
+#ifndef NDEBUG
+	sigset_t tmp;
+	sigprocmask(SIG_BLOCK, NULL, &tmp);
+	assert(sigismember(&tmp, SIGCHLD));
+#endif
+
 	fd_set fdset;
 	FD_ZERO(&fdset);
 	FD_SET(fd, &fdset);
@@ -648,6 +639,7 @@ bool handle_traps(void)
     /* we reset this before executing signal commands to avoid race */
     any_signal_received = false;
 
+    sigset_t emptyset, origset;
     struct parsestate_T *state = NULL;
     savelaststatus = laststatus;
     for (const signal_T *s = signals; s->no; s++) {
@@ -656,8 +648,14 @@ bool handle_traps(void)
 	    signal_received[i] = false;
 	    wchar_t *command = trap_command[i];
 	    if (command && command[0]) {
-		if (!state)
+		if (!state) {
 		    state = save_parse_state();
+		    sigemptyset(&emptyset);
+		    sigemptyset(&origset);
+		    if (sigprocmask(SIG_SETMASK, &emptyset, &origset) < 0
+			    && errno != EINTR)
+			xerror(errno, "sigprocmask(SETMASK, none)");
+		}
 		handled_signal = s->no;
 		exec_wcs(command, "trap", false);
 		laststatus = savelaststatus;
@@ -675,8 +673,14 @@ bool handle_traps(void)
 	    rtsignal_received[i] = false;
 	    wchar_t *command = rttrap_command[i];
 	    if (command && command[0]) {
-		if (!state)
+		if (!state) {
 		    state = save_parse_state();
+		    sigemptyset(&emptyset);
+		    sigemptyset(&origset);
+		    if (sigprocmask(SIG_SETMASK, &emptyset, &origset) < 0
+			    && errno != EINTR)
+			xerror(errno, "sigprocmask(SETMASK, none)");
+		}
 		handled_signal = sigrtmin + i;
 		exec_wcs(command, "trap", false);
 		laststatus = savelaststatus;
@@ -690,6 +694,8 @@ bool handle_traps(void)
     handled_signal = -1;
     if (state) {
 	restore_parse_state(state);
+	if (sigprocmask(SIG_SETMASK, &origset, NULL) < 0 && errno != EINTR)
+	    xerror(errno, "sigprocmask(SETMASK)");
 	return true;
     } else {
 	return false;
